
package org.ow2.petals.tools.juddiAdmin.connection;

import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.PasswordAuthentication;
import java.util.Collection;
import java.util.HashSet;
import java.util.Iterator;
import java.util.Properties;
import java.util.Set;

import javax.xml.registry.BulkResponse;
import javax.xml.registry.BusinessLifeCycleManager;
import javax.xml.registry.BusinessQueryManager;
import javax.xml.registry.Connection;
import javax.xml.registry.ConnectionFactory;
import javax.xml.registry.JAXRException;
import javax.xml.registry.RegistryService;

import org.apache.ws.scout.registry.ConnectionFactoryImpl;
import org.ow2.petals.tools.webadmin.util.WebConsoleConfigException;
import org.ow2.petals.tools.webadmin.util.WebconsoleConfigHelper;

/**
 * This class permit to connect to a UDDI regitry and initialize the attributes
 * which permit you to communicate with the registry.
 * 
 * @author EL JAI Mohammed
 */

public class ConnectRegistry {
    /** attributes for uddi connexion */
    @SuppressWarnings("unused")
    private String httpProxyHost = "";

    @SuppressWarnings("unused")
    private String httpProxyPort = "";

    @SuppressWarnings("unused")
    private String httpsProxyHost = "";

    @SuppressWarnings("unused")
    private String httpsProxyPort = "";

    private String regUrli = "";

    private String regUrlp = "";

    private String username = "";

    private String password = "";

    /** properties file name */
    private static final String propertiesFile = "webconsole.cfg.xml";

    /** attributes names in config file */
    private static final String JUDDI = "juddi";

    private static final String QUERY_URL = "query-url";

    private static final String PUBLISH_URL = "publish-url";

    private static final String USER_NAME = "user-name";

    private static final String USER_PASSWORD = "user-password";

    private static final String PROXY_HOST = "http-proxy-host";

    private static final String PROXY_PORT = "http-proxy-port";

    /** connexion attribues */
    private Properties connProps = new Properties();

    private Properties publishProps;

    /**
     * A BusinessQueryManager object allows you to search objects into a UDDI
     * registry.
     */
    private BusinessQueryManager bqm;

    /**
     * A BusinessLifeCycleManager object allows you to create objects into a
     * UDDI registry.
     */
    private BusinessLifeCycleManager blm;

    /** the single ConnectRegistry object * */
    private static ConnectRegistry connection;

    /**
     * Return the actual connection.
     * 
     * @param realPath :
     *            the real path of the root directory
     * @return a ConnectRegistry object
     * @throws JAXRException
     * @throws IOException
     * @throws FileNotFoundException
     * @throws WebConsoleConfigException
     */
    public static ConnectRegistry getConnection() throws JAXRException, WebConsoleConfigException {
        if (connection == null)
            connection = new ConnectRegistry();
        return connection;
    }

    /**
     * Private contructor to create the single object ConnectRegistry.
     * 
     * @param path
     *            the path of the root directory
     * @throws IOException
     * @throws FileNotFoundException
     * @throws JAXRException
     * @throws WebConsoleConfigException
     * @throws JAXRException
     */
    private ConnectRegistry() throws WebConsoleConfigException, JAXRException {
        
        setConnectionProperties();
        
        // connect to registry where the paramters given in propreties file
        ConnectionFactory factory = ConnectionFactoryImpl.newInstance();
        factory.setProperties(connProps);
        Connection conn = factory.createConnection();
        PasswordAuthentication passwdAuth = new PasswordAuthentication(username, password
                .toCharArray());
        Set<PasswordAuthentication> creds = new HashSet<PasswordAuthentication>();
        creds.add(passwdAuth);
        conn.setCredentials(creds);

        // initilize the registry communication attributes
        RegistryService rs = conn.getRegistryService();
        bqm = rs.getBusinessQueryManager();
        blm = rs.getBusinessLifeCycleManager();
    }

    /**
     * Assigne connection attributes.
     * @throws WebConsoleConfigException 
     */
    private void setConnectionProperties() throws WebConsoleConfigException {
        /** load and assign properties * */
        WebconsoleConfigHelper.init(propertiesFile);
        httpProxyHost = WebconsoleConfigHelper.getProperty(ConnectRegistry.JUDDI,
                ConnectRegistry.PROXY_HOST);
        httpsProxyHost = httpProxyHost;

        httpProxyPort = WebconsoleConfigHelper.getProperty(ConnectRegistry.JUDDI,
                ConnectRegistry.PROXY_PORT);

        regUrli = WebconsoleConfigHelper.getProperty(ConnectRegistry.JUDDI,
                ConnectRegistry.QUERY_URL);
        regUrlp = WebconsoleConfigHelper.getProperty(ConnectRegistry.JUDDI,
                ConnectRegistry.PUBLISH_URL);
        username = WebconsoleConfigHelper.getProperty(ConnectRegistry.JUDDI,
                ConnectRegistry.USER_NAME);
        password = WebconsoleConfigHelper.getProperty(ConnectRegistry.JUDDI,
                ConnectRegistry.USER_PASSWORD);
        connProps.setProperty("javax.xml.registry.queryManagerURL", regUrli);
        connProps.setProperty("javax.xml.registry.lifeCycleManagerURL", regUrlp);
        connProps.setProperty("publish.url", regUrlp);
        connProps.setProperty("query.url", regUrli);
        connProps.setProperty("user.name", username);
        connProps.setProperty("user.password", password);
        connProps.setProperty("http.proxy.host", httpProxyHost);
        connProps.setProperty("http.proxy.port", httpProxyPort);
    }

    /**
     * Handle all exceptions genarated by the BulkResponse object.
     * 
     * @param br
     * @throws JAXRException
     */
    public void handleJAXRExceptions(BulkResponse br) throws JAXRException {
        System.err.println("One or more JAXRExceptions occurred during the save operation:");
        Collection exceptions = br.getExceptions();
        Iterator iter = exceptions.iterator();
        while (iter.hasNext()) {
            Exception e = (Exception) iter.next();
            System.err.println(e.toString());
        }
    }

    /**
     * @return the blm
     */
    public BusinessLifeCycleManager getBlm() {
        return blm;
    }

    /**
     * @param blm
     *            the blm to set
     */
    public void setBlm(BusinessLifeCycleManager blm) {
        this.blm = blm;
    }

    /**
     * @return the bqm
     */
    public BusinessQueryManager getBqm() {
        return bqm;
    }

    /**
     * @param bqm
     *            the bqm to set
     */
    public void setBqm(BusinessQueryManager bqm) {
        this.bqm = bqm;
    }

}
