/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: MsgsPerSecondDataProd.java msauvage $
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.cewolf.dataproducers;

import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.COMPONENT;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.CONTAINER;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.ENDPOINT_NAME;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.ID;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.ROLE;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.Role.CONSUMER;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.Role.PROVIDER;

import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;

import org.apache.log4j.Logger;
import org.jfree.data.general.DefaultPieDataset;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.util.StringHelper;

import de.laures.cewolf.DatasetProduceException;
import de.laures.cewolf.DatasetProducer;

public final class MsgRepartitionDataProd implements DatasetProducer {

    private static final String actionTitle = "show PEtALS messages repartition chart";

    /**
     * 
     */
    private static final long serialVersionUID = 178454L;

    final Logger log = Logger.getLogger(MsgRepartitionDataProd.class);

    public MsgRepartitionDataProd() {
        super();
    }

    /**
     * Returns the id of the producer
     * 
     * @return an id as a String
     */
    public String getProducerId() {
        return new String("MsgRepartitionDataset Producer");
    }

    /**
     * Indicates if dataset produced has expired. method is called by the Cewolf
     * framework to check if a formerly produced data can be reused. As datas
     * are evolving and retrieved in an MBean, method always returns true to
     * force reloading datas.
     * 
     * @return true if the data which had been produced with the passed in
     *         parameters has expired since its creation, false otherwise
     */
    @SuppressWarnings("unchecked")
    public boolean hasExpired(Map params, Date since) {
        return true;
    }

    @SuppressWarnings("unchecked")
    public Object produceDataset(Map params) throws DatasetProduceException {
        final String server = (String) params.get("server");
        final String component = (String) params.get("component");

        final Map<String, Long> msgRepartitionMap = new HashMap<String, Long>();
        final Map<String, Set<String>> petalsIds = new HashMap<String, Set<String>>();

        try {
            final CollectorServer collector = CollectorManager.getCollector();

            List<Map<String, Object>> exchanges = null;
            if (StringHelper.isNotNullAndNotEmpty(component)) {
                exchanges = collector.getDefaultCollectorClient().getMonitoringService()
                        .getAllMessageExchangesSnapshotsByServerAndComponent(server, component);
                for (final Map<String, Object> mes : exchanges) {
                    final String petalsId = (String) mes.get(ID);
                    final String endpoint = (String) mes.get(ENDPOINT_NAME);
                    final String role = (String) mes.get(ROLE);
                    if (PROVIDER.equalsIgnoreCase(role)) {
                        if ((!petalsIds.containsKey(endpoint))) {
                            petalsIds.put(endpoint, new HashSet<String>());
                        }
                        /** select distinct message exchanges * */
                        if ((!petalsIds.get(endpoint).contains(petalsId))) {
                            petalsIds.get(endpoint).add(petalsId);
                            if (msgRepartitionMap.containsKey(endpoint)) {
                                msgRepartitionMap.put(endpoint,
                                        msgRepartitionMap.get(endpoint) + 1L);
                            } else {
                                msgRepartitionMap.put(endpoint, 1L);
                            }
                        }
                    } else {
                        if ((!petalsIds.containsKey(CONSUMER))) {
                            petalsIds.put(CONSUMER, new HashSet<String>());
                        }
                        /** select distinct message exchanges * */
                        if ((!petalsIds.get(CONSUMER).contains(petalsId))) {
                            petalsIds.get(CONSUMER).add(petalsId);
                            if (msgRepartitionMap.containsKey(CONSUMER)) {
                                msgRepartitionMap.put(CONSUMER,
                                        msgRepartitionMap.get(CONSUMER) + 1L);
                            } else {
                                msgRepartitionMap.put(CONSUMER, 1L);
                            }
                        }
                    }
                }
            } else if (StringHelper.isNotNullAndNotEmpty(server)) {
                exchanges = collector.getDefaultCollectorClient().getMonitoringService()
                        .getAllMessageExchangesSnapshotsByServer(server);
                for (final Map<String, Object> mes : exchanges) {
                    final String petalsId = (String) mes.get(ID);
                    final String comp = (String) mes.get(COMPONENT);
                    if ((!petalsIds.containsKey(comp))) {
                        petalsIds.put(comp, new HashSet<String>());
                    }
                    /** select distinct message exchanges * */
                    if ((!petalsIds.get(comp).contains(petalsId))) {
                        petalsIds.get(comp).add(petalsId);
                        if (msgRepartitionMap.containsKey(comp)) {
                            msgRepartitionMap.put(comp, msgRepartitionMap.get(comp) + 1L);
                        } else {
                            msgRepartitionMap.put(comp, 1L);
                        }
                    }
                }
            } else {
                exchanges = collector.getDefaultCollectorClient().getMonitoringService()
                        .getAllMessageExchangesSnapshots();
                for (final Map<String, Object> mes : exchanges) {
                    final String petalsId = (String) mes.get(ID);
                    final String serverName = "Server " + (String) mes.get(CONTAINER);
                    if ((!petalsIds.containsKey(serverName))) {
                        petalsIds.put(serverName, new HashSet<String>());
                    }
                    /** select distinct message exchanges * */
                    if ((!petalsIds.get(serverName).contains(petalsId))) {
                        petalsIds.get(serverName).add(petalsId);
                        if (msgRepartitionMap.containsKey(serverName)) {
                            msgRepartitionMap.put(serverName,
                                    msgRepartitionMap.get(serverName) + 1L);
                        } else {
                            msgRepartitionMap.put(serverName, 1L);
                        }
                    }
                }
            }

            final DefaultPieDataset ds = new DefaultPieDataset();
            for (final String entity : msgRepartitionMap.keySet()) {
                ds.setValue(entity, msgRepartitionMap.get(entity));
            }

            return ds;

            /**
             * as the exception is managed by the chart, we have to log and
             * throw it*
             */
        } catch (final DataCollectorServerException e) {
            this.log.error("Can't " + MsgRepartitionDataProd.actionTitle, e);
            throw new DatasetProduceException(e.getMessage());
        } catch (final DataCollectorClientException e) {
            this.log.error("Can't " + MsgRepartitionDataProd.actionTitle, e);
            throw new DatasetProduceException(e.getMessage());
        } catch (DataCollectorServerNotInitializedException e) {
            CollectorManager.resetCollector();
            this.log.error("Can't " + MsgRepartitionDataProd.actionTitle, e);
            throw new DatasetProduceException(e.getMessage());
        }
    }

}
