/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.contentrendering;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.util.Properties;

import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.ow2.petals.tools.webadmin.ui.infra.bean.MessageLVO;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * @author gdecarnin - Open Wide, mcarpentier - EBM WebSouricng
 */
public class MessageContentRenderingServiceXsl implements MessageContentRenderingService {
    
    private static String XSLT_FOLDER = "xsl/";
    
    private Properties properties = null;
    
    public MessageContentRenderingServiceXsl() {
        properties = new Properties();
        InputStream inStream = this.getClass().getClassLoader().getResourceAsStream(XSLT_FOLDER + "xslfiles.properties");
        try {
            properties.load(inStream);
            inStream.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
    
    public String display(MessageLVO msg) {
        InputStream xsl = null;
        ByteArrayOutputStream outStream = null;
        try {
            // get XSLT
            //FIXME
            String key = "FIXME" + "." + "FIXME";
            String xsltFileName = (String) properties.get(key);
            if (xsltFileName == null) {
                // content for HTML
                String toDisplay=msg.getContent();
                toDisplay = StringHelper.prettyPrint(toDisplay);
                toDisplay = StringHelper.HTMLEntityEncode(toDisplay);
                return toDisplay;
            }
            
            xsl = this.getClass().getClassLoader().getResourceAsStream(XSLT_FOLDER + xsltFileName);
            StreamSource xslSource = new StreamSource(xsl);
            
            // get XML
            String xml = msg.getContent();
            Source xmlSource = new StreamSource(new StringReader(xml));
            
            // transform
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer(xslSource);
            transformer.setOutputProperty(OutputKeys.INDENT, "no");
            transformer.setOutputProperty(OutputKeys.ENCODING, "UTF-8");
            
            outStream = new ByteArrayOutputStream();
            Result result = new StreamResult(outStream);
            transformer.transform(xmlSource, result);
            
            return outStream.toString();
        } catch (Exception e) {
            return e.getMessage();
        } finally {
            if (xsl != null) {
                try {
                    xsl.close();
                } catch (IOException e) {}
            }
            if (outStream != null) {
                try {
                    outStream.close();
                } catch (IOException e) {}
            }
        }
    }

}
