/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.datacollector;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.validator.DynaValidatorForm;
import org.ow2.petals.jmx.api.IPetalsAdminService.ContainerKeyNames;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * Display all adminsitration stuffs for the datacollector
 * 
 * @author mcarpentier - EBM Websourcing
 * @author cgirodengo - Capgemini Sud
 * 
 */
public class ShowDatacollectorAdmin extends Action {
    @Override
    public ActionForward execute(ActionMapping iActionMapping, ActionForm form,
            HttpServletRequest iRequest, HttpServletResponse iResponse) {
        DynaValidatorForm dcForm = (DynaValidatorForm) form;

        final Logger log = Logger.getLogger(ShowDatacollectorAdmin.class);

        /** deals with error messages * */
        AdminSession adminSession = AdminSession.getAdminSession(iRequest.getSession());
        InfoBean infoBean = adminSession.getInfoBean();

        try {
            if (CollectorManager.isCollectorInitialized()) {
                CollectorServer collector = CollectorManager.getCollector();
                Map<String, String> defaultContainerProperties = collector
                        .getDefaultCollectorClient().getContainerProperties();
                dcForm.set("petalsHost", defaultContainerProperties
                        .get(ContainerKeyNames.CONF_HOST));
                dcForm.set("petalsPort", defaultContainerProperties
                        .get(ContainerKeyNames.CONF_JMX_JNDI_PORT));
                dcForm.set("petalsUsername", defaultContainerProperties
                        .get(ContainerKeyNames.CONF_USER));

                dcForm.set("dbPath", String.valueOf(collector.getDatabasePath()));
                dcForm.set("dbPort", String.valueOf(collector.getDatabasePort()));

                dcForm.set("queuesCollectInterval", String.valueOf(collector
                        .getQueuesCollectingInterval()));
                dcForm.set("reconnectionInterval", String.valueOf(collector
                        .getReconnectionInterval()));
                dcForm.set("reconnectionAttempts", String.valueOf(collector
                        .getReconnectionAttempts()));
            }

        } catch (DataCollectorServerException e) {
            infoBean.setMessage("Error : " + StringHelper.formatExceptionMessage(e.getMessage()));
            infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
            log.error("Error : can't admin datacollector", e);
            return iActionMapping.findForward("failure");
        } catch (DataCollectorServerNotInitializedException e) {
            infoBean.setMessage("Error : " + StringHelper.formatExceptionMessage(e.getMessage()));
            infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
            log.error("Error : can't admin datacollector", e);
            return iActionMapping.findForward("failure");
        }

        return iActionMapping.findForward("success");

    }

}
