/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.juddi;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.xml.registry.JAXRException;
import javax.xml.registry.infomodel.Concept;
import javax.xml.registry.infomodel.Organization;
import javax.xml.registry.infomodel.Service;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.apache.struts.action.DynaActionFormClass;
import org.apache.struts.config.FormBeanConfig;
import org.apache.struts.config.ModuleConfig;
import org.apache.struts.util.ModuleUtils;
import org.apache.struts.validator.DynaValidatorForm;
import org.ow2.petals.tools.juddiAdmin.search.SearchRegistry;
import org.ow2.petals.tools.juddiAdmin.search.SearchRegistryImpl;
import org.ow2.petals.tools.juddiAdmin.update.UpdateRegistry;
import org.ow2.petals.tools.juddiAdmin.update.UpdateRegistryImpl;
import org.ow2.petals.tools.webadmin.ui.infra.action.JuddiAction;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.util.WebConsoleConfigException;

/**
 * Upload a component to a temporary directory and install it
 * 
 * request params : componentFile, server
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class SubmitPetalsService extends JuddiAction {

    private static final String actionTitle = "submit PEtALS service";

    private static String CHANGE_ORGANIZATION_SUBMIT = "1";

    final Logger log = Logger.getLogger(SubmitPetalsService.class);

    @Override
    public ActionForward executeJuddiAuthentified(ActionMapping iActionMapping, ActionForm iActionForm,
            HttpServletRequest iServletRequest, HttpServletResponse iServletResponse) {

        final DynaValidatorForm form = (DynaValidatorForm) iActionForm;
        final AdminSession adminSession = AdminSession
                .getAdminSession(iServletRequest.getSession());

        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.JUDDI_ADMINISTRATION);

        /** get form datas * */
        final String component = (String) form.get("component");
        final String server = (String) form.get("server");
        final String submitType = (String) form.get("submitType");
        try {
            /** show display datas * */
            final ModuleConfig moduleConfig = ModuleUtils.getInstance().getModuleConfig(
                    iServletRequest);
            final FormBeanConfig formConfig = moduleConfig
                    .findFormBeanConfig("juddiAddPetalsServiceDisplay");
            final DynaActionFormClass dynaClass = DynaActionFormClass
                    .createDynaActionFormClass(formConfig);
            final DynaActionForm formD = (DynaActionForm) dynaClass.newInstance();
            if (SubmitPetalsService.CHANGE_ORGANIZATION_SUBMIT.equalsIgnoreCase(submitType)) {
                formD.set("organization2", (String) form.get("organization"));
                iServletRequest.setAttribute("juddiAddPetalsServiceDisplay", formD);
            } else {

                // otherwise user has select an organization, page has to be
                // rebuild

                final String organization = (String) form.get("organization");
                final String serviceName = (String) form.get("service");
                final String bindingName = (String) form.get("bindingName");
                final String bindingAccessUrl = (String) form.get("bindingAccessUrl");
                final String bindingDescription = (String) form.get("bindingDescription");
                final String bindingWsdlUrl = (String) form.get("bindingWsdlUrl");
                final String interfaceWsdlDescription = (String) form
                        .get("interfaceWsdlDescription");

                final SearchRegistry search = new SearchRegistryImpl();
                final UpdateRegistry update = new UpdateRegistryImpl();

                final Organization org = search.findOrganizationByName(organization);
                final Service service = search.findServiceByCompany(serviceName, org);

                // create a Jaxr concept object which groups technical
                // information about a service
                final Concept concept = update.createConcept(serviceName + "Interface",
                        interfaceWsdlDescription, bindingWsdlUrl, null);
                // save a petals service into UDDI registry
                update.createBindingTemplate(bindingName, bindingDescription, bindingAccessUrl,
                        concept, service);

                /** deals with info messages **/
                this.fillInfoBean(adminSession, "PEtALS Service successfully added",
                        InfoBean.SUCCESS_MESSAGE);

                formD.set("component", component);
                formD.set("server", server);
                formD.set("submitType", submitType);

                // reset form fields
                form.initialize(iActionMapping);
                iServletRequest.setAttribute("juddiAddPetalsServiceDisplay", formD);
            }
        } catch (final JAXRException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, SubmitPetalsService.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + SubmitPetalsService.actionTitle, e);
        } catch (final IllegalAccessException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, SubmitPetalsService.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + SubmitPetalsService.actionTitle, e);
        } catch (final InstantiationException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, SubmitPetalsService.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + SubmitPetalsService.actionTitle, e);
        } catch (WebConsoleConfigException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, SubmitPetalsService.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + SubmitPetalsService.actionTitle, e);
        }
        return iActionMapping.findForward("success");

    }
}
