/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.logon;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.Action;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * Triggered when an user logs in
 * 
 * @author mcarpentier
 * @author cgirodengo - Capgemini Sud
 * 
 */
public class Logon extends Action {
    @Override
    public ActionForward execute(ActionMapping iActionMapping, ActionForm form,
            HttpServletRequest iRequest, HttpServletResponse iResponse) {

        final Logger log = Logger.getLogger(Logon.class);

        DynaActionForm logonForm = (DynaActionForm) form;

        String petalsUser = (String) logonForm.get("login");
        String petalsPort = (String) logonForm.get("port");
        String petalsHost = (String) logonForm.get("host");
        String petalsPwd = (String) logonForm.get("password");
        String nextUrl = (String) logonForm.get("nextUrl");

        AdminSession adminSession = AdminSession.getAdminSession(iRequest.getSession());

        /** deals with error messages * */
        InfoBean infoBean = adminSession.getInfoBean();

        try {
            if (!CollectorManager.isCollectorInitialized()) {
                /** will start the embedded HSLQDB server if not already started* */
             //   CollectorServer.startHSQLServer("org.hsqldb.jdbcDriver",9003,"sa","");
                CollectorManager.initCollector(petalsHost, petalsPort, petalsUser, petalsPwd);
                CollectorServer collector = CollectorManager.getCollector();
                collector.checkDatabaseIsRunning();
            }
        } catch (DataCollectorServerException e) {
            infoBean.setMessage("Error : " + StringHelper.formatExceptionMessage(e.getMessage()));
            infoBean.setMessageType(InfoBean.ERROR_MESSAGE);
            log.error("Error : login failure", e);
            CollectorManager.destroyCollector();
            return iActionMapping.findForward("failure");
        }

        if ((nextUrl == null) || (nextUrl.length() == 0)) {
            return iActionMapping.findForward("success");
        }

        return new ActionForward(nextUrl, true);

    }
}
