
package org.ow2.petals.tools.webadmin.util;

import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.COMPONENT;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.CONTAINER;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.CONTENT;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.ENDPOINT_NAME;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.ID;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.MEP;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.OPERATION;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.ROLE;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.SERVICE_NAME;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.STATUS;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.TIME;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.ReportKeyNames.TYPE;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.Role.CONSUMER;
import static org.ow2.petals.jmx.api.IPetalsMonitoringService.Role.PROVIDER;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.management.ObjectName;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.ow2.petals.jmx.api.IPetalsAdminService.ContainerKeyNames;
import org.ow2.petals.jmx.api.IPetalsAdminService.DomainKeyNames;
import org.ow2.petals.jmx.api.IPetalsAdminService.SubDomainKeyNames;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.client.service.MonitoringService;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.bean.ComponentLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.ContainerLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.DomainLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.EndPointLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.MessageExchangeLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.MessageLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.ServiceAssemblyLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.SubdomainLVO;

public class DataCollectorHelper {
    
    /**
     * The logger
     */
    static Logger logger = Logger.getLogger(DataCollectorHelper.class);

    /**
     * retrieve all PEtALS configurations via JMX
     * 
     * @return
     * @throws DataCollectorServerException
     * @throws DataCollectorClientException
     * @throws DataCollectorServerNotInitializedException
     * @throws DataCollectorClientDoesNotExistException
     */
    public static DomainLVO updateDomainInfo(boolean refresh) throws DataCollectorServerException,
            DataCollectorClientException, DataCollectorServerNotInitializedException,
            DataCollectorClientDoesNotExistException {

        final CollectorServer collector = CollectorManager.getCollector();
        if (refresh) {
            collector.initServer(true);
        }

        final Map<String, String> domainProperties = collector.getDomainProperties();
        if (logger.isDebugEnabled()) {
            logger.debug("domainProperties:" + domainProperties);
        }
        final DomainLVO domainLVO = new DomainLVO();
        domainLVO.setMode(domainProperties.get(DomainKeyNames.CONF_DOMAIN_MODE));
        domainLVO.setName(domainProperties.get(DomainKeyNames.CONF_DOMAIN_NAME));
        domainLVO.setDescription(domainProperties.get(DomainKeyNames.CONF_DOMAIN_DESCRIPTION));
        domainLVO.setMulticastIP(domainProperties.get(DomainKeyNames.CONF_NETWORK_MULTICAST_IP));
        domainLVO
                .setMulticastPort(domainProperties.get(DomainKeyNames.CONF_NETWORK_MULTICAST_PORT));

        final Set<Map<String, String>> subdomainsProperties = collector.getSubdomainsProperties();
        if (logger.isDebugEnabled()) {
            logger.debug("subdomainsProperties:" + subdomainsProperties);
        }
        for (Map<String, String> subdomainProperties : subdomainsProperties) {
            final SubdomainLVO subdomainLVO = new SubdomainLVO();
            subdomainLVO.setName(subdomainProperties.get(SubDomainKeyNames.CONF_SUBDOMAIN_NAME));
            subdomainLVO.setDescription(subdomainProperties
                    .get(SubDomainKeyNames.CONF_SUBDOMAIN_NAME));
            subdomainLVO.setNetworkSynchronized(Boolean.valueOf(subdomainProperties
                    .get(SubDomainKeyNames.CONF_NETWORK_TIME_SYNCHRONIZED)));
            domainLVO.addSubdomain(subdomainLVO);
        }

        final Set<Map<String, String>> containersProperties = collector.getContainersProperties();
        if (logger.isDebugEnabled()) {
            logger.debug("containersProperties:" + containersProperties);
        }

        /** retrieve all endpoints * */
        if (containersProperties.size() > 0) {

            /** retrieve endpoints* */
            final List<Map<String, Object>> endpoints = collector.getDefaultCollectorClient()
                    .getEndpointService().getAllEndpoints();
            final List<EndPointLVO> epsLVO = new ArrayList<EndPointLVO>();
            for (final Map<String, Object> enpoint : endpoints) {
                final String[] interfaceNamesString = (String[]) enpoint.get("interfaceNames");
                String interfaceNames = "";
                for (final String name : interfaceNamesString) {
                    if (StringHelper.isNullOrEmpty(interfaceNames)) {
                        interfaceNames = QName.valueOf(name).toString();
                    } else {
                        interfaceNames = interfaceNames + "," + QName.valueOf(name);
                    }
                }
                final String endpointName = (String) enpoint.get(ENDPOINT_NAME);
                final String type = (String) enpoint.get(TYPE);
                final String serviceNameString = (String) enpoint.get(SERVICE_NAME);
                final QName serviceName = QName.valueOf(serviceNameString);
                /** TODO: make petals kernel service use jmx api * */
                final String containerName = (String) enpoint.get("containerName");
                final String componentName = (String) enpoint.get("componentName");
                final EndPointLVO epLVO = new EndPointLVO();
                epLVO.setComponent(componentName);
                epLVO.setServer(containerName);
                epLVO.setInterfaceName(interfaceNames);
                epLVO.setName(endpointName);
                epLVO.setService(serviceName.toString());
                epLVO.setType(type);
                epsLVO.add(epLVO);
            }

            /** retrieve all components and SA* */
            for (final Map<String, String> containerProperties : containersProperties) {
                String servName = containerProperties.get(ContainerKeyNames.CONF_CONTAINER_NAME);

                final ContainerLVO servLVO = new ContainerLVO(servName);
                servLVO.setState(containerProperties.get(ContainerKeyNames.CONF_STATE));
                servLVO.setHost(containerProperties.get(ContainerKeyNames.CONF_HOST));
                servLVO.setDescription(containerProperties.get(ContainerKeyNames.CONF_CONTAINER_DESCRIPTION));
                servLVO.setState(containerProperties.get(ContainerKeyNames.CONF_STATE));
                servLVO.setJmxPort(containerProperties.get(ContainerKeyNames.CONF_JMX_JNDI_PORT));
                servLVO.setNetworkBindPort(containerProperties
                        .get(ContainerKeyNames.CONF_NETWORK_BIND_PORT));
                servLVO.setDreamTCPPort(containerProperties.get(ContainerKeyNames.CONF_DREAM_TCP_PORT));
                servLVO.setJoramDomainPort(containerProperties
                        .get(ContainerKeyNames.CONF_JORAM_DOMAIN_PORT));
                servLVO.setJoramTCPPort(containerProperties.get(ContainerKeyNames.CONF_JORAM_TCP_PORT));

                /** retrieve service assemblies * */
                final String[] sas = collector.getCollectorClient(servName).getDeployService()
                        .getServiceAssemblies();

                for (final String sa : sas) {
                    final String saState = collector.getCollectorClient(servName)
                            .getDeployService().getServiceAssemblyState(sa);
                    servLVO.addServiceAssembly(new ServiceAssemblyLVO(sa, saState));

                }

                /** retrieve shared libraries * */
                final String[] sls = collector.getCollectorClient(servName).getInstallService()
                        .retrieveInstalledSharedLibraries();

                for (final String sl : sls) {
                    servLVO.addSharedLibrary(sl);
                }

                ObjectName[] objComps = collector.getCollectorClient(servName).getAdminService()
                        .getBindingComponents();

                for (final ObjectName objectName : objComps) {
                    final String name = objectName.getKeyProperty("name");
                    final String objState = collector.getCollectorClient(servName)
                            .getAdminService().getComponentState(name);
                    final ComponentLVO cLVO = new ComponentLVO(name, objState);
                    cLVO.setType(ComponentLVO.BC_TYPE);
                    cLVO.setObjectName(objectName.getCanonicalName());
                    for (final EndPointLVO epLVO : epsLVO) {
                        if ((cLVO.getName().equalsIgnoreCase(epLVO.getComponent()))
                                && (servName.equalsIgnoreCase(epLVO.getServer()))) {
                            cLVO.addEndpoint(epLVO);
                        }
                    }
                    servLVO.addComponent(cLVO);

                }
                objComps = collector.getCollectorClient(servName).getAdminService()
                        .getEngineComponents();

                for (final ObjectName objectName : objComps) {
                    final String name = objectName.getKeyProperty("name");
                    final String objState = collector.getCollectorClient(servName)
                            .getAdminService().getComponentState(name);
                    final ComponentLVO cLVO = new ComponentLVO(name, objState);
                    cLVO.setType(ComponentLVO.SE_TYPE);
                    cLVO.setObjectName(objectName.getCanonicalName());
                    for (final EndPointLVO epLVO : epsLVO) {
                        if ((cLVO.getName().equalsIgnoreCase(epLVO.getComponent()))
                                && (servName.equalsIgnoreCase(epLVO.getServer()))) {
                            cLVO.addEndpoint(epLVO);
                        }
                    }
                    servLVO.addComponent(cLVO);

                }

                SubdomainLVO subdomainLVO = domainLVO.getSubdomains().get(
                        containerProperties.get(ContainerKeyNames.CONF_SUBDOMAIN_NAME));
                subdomainLVO.addServer(servLVO);
            }

        }
        return domainLVO;
    }

    public static List<MessageExchangeLVO> getExchanges(String statusFilter) throws DataCollectorServerException,
            DataCollectorClientException, DataCollectorServerNotInitializedException {

        final CollectorServer collector = CollectorManager.getCollector();
        
        final MonitoringService monitoringService = collector.getDefaultCollectorClient()
        .getMonitoringService();
        
        List<Map<String, Object>> exchanges = null;
        
        if (statusFilter.equals(MessageExchangeLVO.ALL)){
            exchanges = monitoringService.getAllMessageExchanges();
        } else {
            exchanges = monitoringService.getMessageExchanges(statusFilter);    
        }
        
        final Map<String, MessageExchangeLVO> exchangesList = new HashMap<String, MessageExchangeLVO>();

        if (exchanges != null) {

            for (final Map<String, Object> re : exchanges) {

                final String meId = (String) re.get(ID);
                MessageExchangeLVO currentMe;
                if (exchangesList.containsKey(meId)) {
                    currentMe = exchangesList.get(meId);
                } else {
                    currentMe = new MessageExchangeLVO();
                    currentMe.setPetalsId(meId);
                    exchangesList.put(meId, currentMe);
                }

                proceedExchange(currentMe, re);

            }
        }

        return new ArrayList<MessageExchangeLVO>(exchangesList.values());

    }

    public static MessageExchangeLVO getExchange(String exchangeId)
            throws DataCollectorServerException, DataCollectorClientException,
            DataCollectorServerNotInitializedException {

        final CollectorServer collector = CollectorManager.getCollector();

        final List<Map<String, Object>> exchanges = collector.getDefaultCollectorClient()
                .getMonitoringService().getMessageExchange(exchangeId);

        final MessageExchangeLVO result = new MessageExchangeLVO();

        if (exchanges != null) {
            result.setPetalsId(exchangeId);
            for (final Map<String, Object> re : exchanges) {
                final String meId = (String) re.get(ID);

                proceedExchange(result, re);

            }
        }

        return result;

    }

    private static void proceedExchange(MessageExchangeLVO currentMe, Map<String, Object> m) {

        String meType = (String) m.get(TYPE);
        String meStatus = (String) m.get(STATUS);
        String finalMeType;
        if ((MessageLVO.DONE.equalsIgnoreCase(meStatus))
                || (MessageLVO.ERROR.equalsIgnoreCase(meStatus))) {
            finalMeType = meStatus;
        } else {
            finalMeType = meType;
        }

        String meRole = (String) m.get(ROLE);
        String meComponent = (String) m.get(COMPONENT);
        long meTime = ((Long) m.get(TIME)).longValue();

        /** set ME "constant" properties if not already set* */
        if (StringHelper.isNullOrEmpty(currentMe.getServer())) {
            currentMe.setOperation((String) m.get(OPERATION));
            currentMe.setPattern((String) m.get(MEP));
        }
        if ((PROVIDER.equalsIgnoreCase(meRole))
                && (StringHelper.isNullOrEmpty(currentMe.getProvider()))) {
            currentMe.setProvider(meComponent);
            /** update me's properties * */
            currentMe.setServer((String) m.get(CONTAINER));
            currentMe.setService((String) m.get(SERVICE_NAME));
            currentMe.setEndpoint((String) m.get(ENDPOINT_NAME));

        } else if ((CONSUMER.equalsIgnoreCase(meRole))
                && (StringHelper.isNullOrEmpty(currentMe.getConsumer()))) {
            currentMe.setConsumer(meComponent);
        }
        if (meTime < currentMe.getStartTime()) {
            currentMe.setStartTime(meTime);
        }
        if (meTime >= currentMe.getEndTime()) {

            currentMe.setEndTime(meTime);
        }
        if ((!MessageLVO.DONE.equalsIgnoreCase(currentMe.getStatus()))
                && (!MessageLVO.ERROR.equalsIgnoreCase(currentMe.getStatus()))) {
            currentMe.setStatus(meStatus);
        }
        currentMe.addInvolvedComponent(meComponent);

        /** deals with exchanges list* */
        if (currentMe.getExchanges() == null) {
            currentMe.setExchanges(new HashMap<String, MessageLVO>());
        }

        MessageLVO exLVO;
        if (currentMe.getExchanges().containsKey(finalMeType)) {
            /** one of the two expected snapshots has already been registered * */
            exLVO = currentMe.getExchanges().get(finalMeType);
            if (meTime > exLVO.getEndTime()) {
                exLVO.setEndTime(meTime);
            } else if (meTime < exLVO.getStartTime()) {
                exLVO.setStartTime(meTime);
            }

        } else {
            exLVO = new MessageLVO();
            exLVO.setContent((String) m.get(CONTENT));
            exLVO.setStartTime(((Long) m.get(TIME)).longValue());
            exLVO.setEndTime(((Long) m.get(TIME)).longValue());
            exLVO.setType(finalMeType);
            currentMe.addExchange(exLVO);
        }

    }

}
